BINDING_HEADER_STACKPACK_BINDING_HEADER = "Stackpack"
local Version = "10.0.2a"
Stackpack = { Version=Version }

local OlderThan
local function WhatsNew()
	local prevVersion = StackpackSettings["Version"] or ""
	StackpackSettings["Version"] = Version
	local text = ""
	if OlderThan(prevVersion, "10.0.2a") then
		text = text .. "\n\n|cffffffff[10.0.2a]\n|cff646436"..
			"Due to changes in how settings are stored,\n" ..
			"Stackpack settings were reset to defaults.\n|cffA0A080" ..
			"Please check and update your settings -\n" ..
			"use |cffffffff/stackpack|cffA0A080 for more info!\n\n|cff646436" ..
			"By popular demand, the search button is now movable! (SHIFT-drag it!)\n\n" ..
			"Stack sizes are now available in tooltips!"
	end
	if strlen(text) < 1 then return end

	StaticPopupDialogs["STACKPACK_WHATS_NEW"] = {
		text = "What's New in Stackpack" .. text,
		button1 = "Got It!",
		timeout = 0,
		whileDead = 1,
		hideOnEscape = 1,
		wide = 1,
	}
	StaticPopup_Show("STACKPACK_WHATS_NEW")
end

local DefaultSettings = {
	["StackGuildBank"] = true,
	["UseInventorySearch"] = true,
	["ShowItemIDs"] = false,
	["ShowIlvls"] = false,
	["ShowStacksizes"] = true
}

local QueryScan
local StackItem
local Enabled = 1
local Queued = -1
local ForceScan = 0
local Stacking = 0
local SPCycle = 0
local SPToggle = 0
local BankOpen = 0
local GuildBankOpen = 0
local GuildBankDelay = 0
local GuildBankStackAll = 0
local ReadyID = 0
local lastReady = -1
local State = 0
local Queue = {}
local Ready = {}
local StackHook = nil
local SplitStack = 0
local WFrame = getglobal("WorldFrame")
local GuildBankStackButton = nil
local PBIndex = 0
local isLooting = false
local TTHooked = false
local cantDE = {
	["5004"] = 1,["11287"] = 1,["11288"] = 1,["11289"] = 1,["11290"] = 1,["12772"] = 1,
	["14812"] = 1,["18665"] = 1,["20406"] = 1,["20407"] = 1,["20408"] = 1,["21766"] = 1,
	["29378"] = 1,["31336"] = 1,["32540"] = 1,["32541"] = 1,["32660"] = 1,["32662"] = 1,
	["49121"] = 1,["49123"] = 1,["49124"] = 1,["49126"] = 1
}
local cantDESet = {
	-- Alliance sets missing
	-- WOW these need to be updated! ;)
	"Warlord's","Blood Guard's","General's","Sergeant's","Guardian's","Vindicator's","Gladiator's",
	" of the Horde"
}
local cantDF = function(ID)
	local Item = GetItemInfo(ID)
	for c=1, #cantDESet do
		if (strfind(Item,cantDESet[c]) ~= nil) then return 1 end
	end
	return 0
end

-- These used to be global from Blizzard_GuildBankUI.lua but are now "local" there as of 9.1.5
local MAX_GUILDBANK_SLOTS_PER_TAB = MAX_GUILDBANK_SLOTS_PER_TAB or 98
local NUM_SLOTS_PER_GUILDBANK_GROUP = NUM_SLOTS_PER_GUILDBANK_GROUP or 14

-- Use official (as of Dragonflight) names for things where known
local REAGENTBANK_CONTAINER = REAGENTBANK_CONTAINER or -3
local BANK_CONTAINER = BANK_CONTAINER or -1
local BACKPACK_CONTAINER = BACKPACK_CONTAINER or 0
local NUM_REAGENTBAG_SLOTS = NUM_REAGENTBAG_SLOTS or 0
local NUM_BAG_SLOTS = NUM_BAG_SLOTS or 4
local NUM_TOTAL_EQUIPPED_BAG_SLOTS = NUM_TOTAL_EQUIPPED_BAG_SLOTS or (NUM_BAG_SLOTS + NUM_REAGENTBAG_SLOTS)
local NUM_BANKGENERIC_SLOTS = NUM_BANKGENERIC_SLOTS or 28

-- Starting and ending bag slot numbers for bank bags
local BANK_BAG_1 = NUM_TOTAL_EQUIPPED_BAG_SLOTS + 1
local BANK_BAG_N = NUM_TOTAL_EQUIPPED_BAG_SLOTS + GetNumBankSlots()


local ttscanmax = 8	-- only search this many lines at most for text

local function ttscan(item, text, where)
	local found = nil
	if not item then return nil end
	local ttdata
	if type(item) == "table" then
		local IN = strlower(tostring(item[1]))
		if IN == "bag" then
			ttdata = C_TooltipInfo.GetBagItem(item[2],item[3])
		elseif IN == "gbank" or IN == "guild" then
			ttdata = C_TooltipInfo.GetGuildBankItem(item[2],item[3])
		end
	else
		ttdata = C_TooltipInfo.GetHyperlink(item)
	end
	TooltipUtil.SurfaceArgs(ttdata)
	for i = 1, ttscanmax do
		if not ttdata.lines[i] then break end
		TooltipUtil.SurfaceArgs(ttdata.lines[i])
		if not where or strlower(where)~="right" then
			local t = ttdata.lines[i].leftText
			  -- look only at patterns, not at what they make:
			if not t or t == "" then break end
			if t:find(text) then found=t; break end
		end
		if not where or strlower(where)~="left" then
			local t = ttdata.lines[i].rightText
			if t and t:find(text) then found=t; break end
		end
	end
	return found
end

local FilterTypes = {
	{
		Type = "eq,equip,equippable", Require = function(ID)
			local _, _, _, _, _, itemType = GetItemInfo(ID)
			if (itemType == ENCHSLOT_WEAPON) or (itemType == ARMOR) then return true end
			return false
		end
	},
	{
		Type = "weap,weapon,weapons", Require = function(ID)
			local _, _, _, _, _, itemType = GetItemInfo(ID)
			if (itemType == ENCHSLOT_WEAPON) then return true end
			return false
		end
	},
	{
		Type = "arm,armor", Require = function(ID)
			local _, _, _, _, _, itemType = GetItemInfo(ID)
			if (itemType == ARMOR) then return true end
			return false
		end
	},
	{
		Type = "boe,bindonequip", Require = function(ID,bag,slot,mode)
			local location="bag"
			if (mode == "GB") then location="gbank" end
			return ttscan({location,bag,slot}, ITEM_BIND_ON_EQUIP, "left") ~= nil
		end
	},
	{
		Type = "bop,soulbound", Require = function(ID,bag,slot,mode)
			if (mode == "GB") then return false end
			return ttscan({"bag",bag,slot}, ITEM_SOULBOUND, "left") ~= nil
		end
	},
	{
		Type = "boa,accountbound,account", Require = function(ID,bag,slot,mode)
			if (mode == "GB") then return false end
			return ttscan({"bag",bag,slot}, ITEM_BNETACCOUNTBOUND, "left") ~= nil
			   and ttscan({"bag",bag,slot}, ITEM_ACCOUNTBOUND, "left") ~= nil
		end
	},
	{
		Type = "gr[ae]y,junk,vendor,trash,poor", Require = function(ID)
			local _, _, Quality = GetItemInfo(ID)
			return Quality == 0
		end
	},
	{
		Type = "W,C,white,common", Require = function(ID)
			local _, _, Quality = GetItemInfo(ID)
			return Quality == 1
		end
	},
	{
		Type = "G,U,green,uncommon", Require = function(ID)
			local _, _, Quality = GetItemInfo(ID)
			return Quality == 2
		end
	},
	{
		Type = "B,R,blue,rare", Require = function(ID)
			local _, _, Quality = GetItemInfo(ID)
			return Quality == 3
		end
	},
	{
		Type = "P,E,purple,epic", Require = function(ID)
			local _, _, Quality = GetItemInfo(ID)
			return Quality == 4
		end
	},
	{
		Type = "O,L,orange,leg,legend,legendary", Require = function(ID)
			local _, _, Quality = GetItemInfo(ID)
			return Quality == 5
		end
	},
	{
		Type = "A,art,artifact", Require = function(ID)
			local _, _, Quality = GetItemInfo(ID)
			return Quality == 6
		end
	},
	{
		Type = "H,heir,loom,heirloom", Require = function(ID)
			local _, _, Quality = GetItemInfo(ID)
			return Quality == 7
		end
	},
	{
		Type = "gold", Require = function(ID)
			local _, _, Quality = GetItemInfo(ID)
			return Quality == 6 or Quality == 7
		end
	},
	{
		Type = "de,deable,disenchant,disenchantable", Require = function(ID)
			if (cantDE[ID] == 1 or cantDF(ID) == 1) then return false end
			local _, _, Quality, _, _, itemType = GetItemInfo(ID)
			if ((itemType == ARMOR or itemType == ENCHSLOT_WEAPON) and Quality > 1 and Quality < 5) then
				return true
			end
			return false
		end
	},
	{	Type = "ilvl,il,  or ilvl query: e.g. ilvl<=200 (all comparisons supported)", Require = function(ID,bag,slot,mode,query)
			local ilvl = ttscan({"bag",bag,slot}, "Item Level", "left")
			if ilvl == nil then return false end
			ilvl = tonumber(strsub(ilvl,12)) or tonumber(strsub(ilvl,13))	-- support "Item Level: " from other addons
			local p, n
			p=query:find(">="); if p then n=tonumber(strsub(query,p+2)); return n and (ilvl >= n); end
			p=query:find("<="); if p then n=tonumber(strsub(query,p+2)); return n and (ilvl <= n); end
			p=query:find("=="); if p then n=tonumber(strsub(query,p+2)); return n and (ilvl == n); end
			p=query:find("!="); if p then n=tonumber(strsub(query,p+2)); return n and (ilvl ~= n); end
			p=query:find("~="); if p then n=tonumber(strsub(query,p+2)); return n and (ilvl ~= n); end
			p=query:find("<>"); if p then n=tonumber(strsub(query,p+2)); return n and (ilvl ~= n); end
			p=query:find(">" ); if p then n=tonumber(strsub(query,p+1)); return n and (ilvl >  n); end
			p=query:find("<" ); if p then n=tonumber(strsub(query,p+1)); return n and (ilvl <  n); end
			p=query:find("=" ); if p then n=tonumber(strsub(query,p+1)); return n and (ilvl == n); end
			return (ilvl>0)
		end
	}
}

local function GetItemID(link)
	if (link == nil) then return nil end
	local _, _, Buffer = strfind(link,"item:(%d+):")
	return Buffer
end

local function GetItemLvl(link)
	if (link == nil) then return nil end
	local _, _, _, ilvl = GetItemInfo(link)
	return ilvl
end

local function ModifyTooltip(tooltip, data)
	local itemlink = tooltip and tooltip.GetItem and select(2, tooltip:GetItem())
	if not itemlink then return end
	local id = StackpackSettings["ShowItemIDs"] and GetItemID(itemlink)
	local ilvl = StackpackSettings["ShowIlvls"] and GetItemLvl(itemlink)
	local stacksize = StackpackSettings["ShowStacksizes"] and select(8, GetItemInfo(itemlink))
	if id or ilvl then
		id = id and "Item ID: |cffffffff"..id
		ilvl = ilvl and "Item Lvl: |cffffffff"..ilvl
		if id and ilvl then
			tooltip:AddDoubleLine(id, ilvl, NORMAL_FONT_COLOR.r, NORMAL_FONT_COLOR.g, NORMAL_FONT_COLOR.b)
		else
			local what = id or ilvl
			tooltip:AddLine(what, NORMAL_FONT_COLOR.r, NORMAL_FONT_COLOR.g, NORMAL_FONT_COLOR.b)
		end
		tooltip:Show()
	end
	if stacksize and stacksize > 1 then
		stacksize = "Stack Size: |cffffffff"..stacksize
		tooltip:AddLine(stacksize, NORMAL_FONT_COLOR.r, NORMAL_FONT_COLOR.g, NORMAL_FONT_COLOR.b)
		tooltip:Show()
	end
end

local function HookTooltips()
	if not TTHooked then
		TooltipDataProcessor.AddTooltipPostCall(Enum.TooltipDataType.Item, ModifyTooltip)
	end
	TTHooked = true
end

local function vercmp(a,b)
	if (a == b) then return 0 end
	local A = { strmatch(a, "(%d*)(%D*)%.(%d*)(%D*)%.(%d*)(%D*)") }
	local B = { strmatch(b, "(%d*)(%D*)%.(%d*)(%D*)%.(%d*)(%D*)") }
	local i = 0
	while true do
		i = i + 1
		if not A[i] and not B[i] then return 0 end
		if i % 2 > 0 then local d = (A[i] or 0) - (B[i] or 0); if d ~= 0 then return d end end
		if (A[i] or "") > (B[i] or "") then return  1 end
		if (A[i] or "") < (B[i] or "") then return -1 end
	end
end

function OlderThan(v,x)	-- is local (forward declared)
	return vercmp(v,x) < 0
end

local function Push(msg,r,g,b)
	if (r == nil or g == nil or b == nil) then
		r = 0.74
		g = 0.74
		b = 0.36
	end
	DEFAULT_CHAT_FRAME:AddMessage(msg,r,g,b)
end

local function showStackWarning()
	StaticPopupDialogs["STACKPACK_STACK_GUILD_BANK"] = {
		text = "|cffff0000WARNING!\n\n|cffffffffIf you continue, your ENTIRE guild bank will be stacked. That means tabs 1-"..GetNumGuildBankTabs().." will have their contents combined.\n\nAre you absolutely sure you want to do this?",
		button1 = YES,
		button2 = NO,
		timeout = 0,
		whileDead = 0,
		hideOnEscape = 1,
		OnAccept = function()
			GuildBankOpen = 1
			GuildBankStackAll = 1
			QueryScan()
		end,
		OnCancel = function()
			GuildBankOpen = 1
			GuildBankStackAll = 0
		end
	}
	StaticPopup_Show("STACKPACK_STACK_GUILD_BANK")
end

local function ApplyFilters(ID,filters,bag,slot,mode)
	if (ID == nil or filters == nil or #filters == 0) then return true end
	local triggerFound = false
	for f=1, #filters do
		local wanted = true
		local filt = filters[f]
		if (filt:find("!")==1) or (filt:find("~")==1) then
			filt = strsub(filt,2)
			wanted = false
		end
		for t=1, #FilterTypes do
			local triggers = { strsplit(",",FilterTypes[t].Type) }
			for w=1, #triggers do
				if (filt:find(triggers[w]) == 1) then
					triggerFound = true
					local filterOK = FilterTypes[t].Require(ID,bag,slot,mode,filters[f])
					if (filterOK ~= wanted) then return false end
					break
				end
			end
		end
	end
	return triggerFound
end

local function BagButtonName(bag)
	if (bag == BACKPACK_CONTAINER) then return "MainMenuBarBackpackButton" end
	if (bag <= NUM_BAG_SLOTS) then return "CharacterBag"..(bag-1).."Slot" end
	return "CharacterReagentBag"..(bag-NUM_BAG_SLOTS-1).."Slot"
end

local function UnfilterBags()
	for t=1,16 do
		local frame = _G["ContainerFrame"..t]
		if (frame ~= nil) then
			for c=1,32 do
				local button = _G["ContainerFrame"..t.."Item"..c]
				if (button ~= nil) then
					button:SetAlpha(1)
				end
			end
		end
	end
	for t=BACKPACK_CONTAINER,NUM_TOTAL_EQUIPPED_BAG_SLOTS do
		local bag = _G[BagButtonName(t)]
		if (bag.matches) then bag.matches:SetText("") end
	end
	local dialog = StaticPopup_FindVisible("STACKPACK_SEARCH_BAGS")
	dialog.text:SetText("Enter a query of 3 or more characters in length")
end

local function FilterBags(query)
	local text = query
	if (text == "Search..." or strlen(text) < 3) then
		UnfilterBags()
		return
	end
	local filters = {}
	while (1) do
		local a, b, filter = strfind(text,"%%([^%%]+)%%")
		if (filter == nil) then break end
		table.insert(filters,filter)
		text = strsub(text,1,a-1)..strsub(text,b+1)
	end
	if (strfind(text,"%%")) then text = gsub(text,"%%","") end
	local results = {}
	local bagResults = {}
	local matches = 0
	for t=BACKPACK_CONTAINER,NUM_TOTAL_EQUIPPED_BAG_SLOTS do
		local bagMatches = 0
		bagResults[t] = {}
		local slots = C_Container.GetContainerNumSlots(t)
		if (slots > 0) then
			for c=1,slots do
				local ID = GetItemID(C_Container.GetContainerItemLink(t,c))
				if (ID ~= nil) then
					local Item, Link, Quality, ItemLevel, RequiredLevel, itemType, otherType, Stack, equipLoc, Texture, Price = GetItemInfo(ID)
					if (strfind(strlower(Item or ""),"(.*)"..strlower(text).."(.*)") ~= nil) then
						local filterOK = ApplyFilters(ID,filters,t,c)
						if (filterOK) then
							matches = matches + 1
							bagMatches = bagMatches + 1
							bagResults[t][slots-(c-1)] = 1
						end
					end
				end
			end
			local bag = _G[BagButtonName(t)]
			if (not bag.matches) then
				bag.matches = bag:CreateFontString(nil, "OVERLAY")
				bag.matches:SetPoint("CENTER", bag, 0, 14)
				bag.matches:SetJustifyH("LEFT")
				bag.matches:SetFontObject(GameFontNormal)
				bag.matches:SetTextHeight(24)
				bag.matches:SetTextColor(0,1,0)
			end
			if (bagMatches == 0) then bagMatches = "" end
			bag.matches:SetText(bagMatches)
		end
	end
	local dialog = StaticPopup_FindVisible("STACKPACK_SEARCH_BAGS")
	if (matches == 0) then
		UnfilterBags()
		dialog.text:SetText("Nothing found for '"..query.."'")
		return
	end
	for t=1,16 do
		local frame = _G["ContainerFrame"..t]
		if (frame ~= nil) then
			local ID = frame:GetID()
			if (ID ~= nil and ID >= 0 and ID <= 4) then
				for c=1,C_Container.GetContainerNumSlots(ID) do
					local button = _G["ContainerFrame"..t.."Item"..c]
					if (button ~= nil) then
						if (bagResults[ID][c] == 1) then
							button:SetAlpha(1)
						else
							button:SetAlpha(0.15)
						end
					end
				end
			end
		end
	end
	dialog.text:SetText("Found " .. matches .. " item(s)")
end

local function UnfilterBank()
	for c=1,NUM_BANKGENERIC_SLOTS do
		local button = _G["BankFrameItem"..c]
		button:SetAlpha(1)
	end
	for t=1,16 do
		local frame = _G["ContainerFrame"..t]
		if (frame ~= nil) then
			for c=1,40 do
				local button = _G["ContainerFrame"..t.."Item"..c]
				if (button ~= nil) then
					button:SetAlpha(1)
				end
			end
		end
	end
	for t=1,7 do
		local bag = _G["BankSlotsFrame"]["Bag"..t]
		if (bag.matches) then bag.matches:SetText("") end
	end
end

local function FilterBank()
	if (BankOpen ~= 1) then return end
	local text = BankFrame.SearchText:GetText()
	if (text == nil) then text="" end
	if (text == "Search..." or strlen(text) < 3) then
		if (strlen(text) > 0 and strlen(text) < 3) then
			BankFrame.matches:SetText("Search too narrow")
		else
			BankFrame.matches:SetText("")
		end
		UnfilterBank()
		return
	end
	local filters = {}
	while (1) do
		local a, b, filter = strfind(text,"%%([^%%]+)%%")
		if (filter == nil) then break end
		table.insert(filters,filter)
		text = strsub(text,1,a-1)..strsub(text,b+1)
	end
	if (strfind(text,"%%")) then text = gsub(text,"%%","") end
	local results = {}
	local bagResults = {}
	local matches = 0
	for c=1,NUM_BANKGENERIC_SLOTS do
		local ID = GetItemID(C_Container.GetContainerItemLink(BANK_CONTAINER,c))
		if (ID ~= nil) then
			local Item, Link, Quality, ItemLevel, RequiredLevel, itemType, otherType, Stack, equipLoc, Texture, Price = GetItemInfo(ID)
			if (strfind(strlower(Item),"(.*)"..strlower(text).."(.*)") ~= nil) then
				local filterOK = ApplyFilters(ID,filters,BANK_CONTAINER,c)
				if (filterOK) then
					matches = matches + 1
					results[c] = 1
				end
			end
		end
	end
	for t=BANK_BAG_1,BANK_BAG_N do
		local bagMatches = 0
		bagResults[t] = {}
		local slots = C_Container.GetContainerNumSlots(t)
		if (slots > 0) then
			for c=1,slots do
				local ID = GetItemID(C_Container.GetContainerItemLink(t,c))
				if (ID ~= nil) then
					local Item, Link, Quality, ItemLevel, RequiredLevel, itemType, otherType, Stack, equipLoc, Texture, Price = GetItemInfo(ID)
					if (strfind(strlower(Item),"(.*)"..strlower(text).."(.*)") ~= nil) then
						local filterOK = ApplyFilters(ID,filters,t,c)
						if (filterOK) then
							matches = matches + 1
							bagMatches = bagMatches + 1
							bagResults[t][slots-(c-1)] = 1
						end
					end
				end
			end
			local bag = _G["BankSlotsFrame"]["Bag"..(t-4)]
			if (not bag.matches) then
				bag.matches = bag:CreateFontString(nil, "OVERLAY")
				bag.matches:SetPoint("CENTER", bag, 0, 14)
				bag.matches:SetJustifyH("LEFT")
				bag.matches:SetFontObject(GameFontNormal)
				bag.matches:SetTextHeight(24)
				bag.matches:SetTextColor(0,1,0)
			end
			if (bagMatches == 0) then bagMatches = "" end
			bag.matches:SetText(bagMatches)
		end
	end
	if (matches == 0) then
		BankFrame.matches:SetText("No results")
		UnfilterBank()
		return
	end
	BankFrame.matches:SetText(matches.." results")
	for c=1,NUM_BANKGENERIC_SLOTS do
		local button = _G["BankFrameItem"..c]
		if (results[c] ~= 1) then
			button:SetAlpha(0.15)
		else
			button:SetAlpha(1)
		end
	end
	for t=1,16 do
		local frame = _G["ContainerFrame"..t]
		if (frame ~= nil) then
			local ID = frame:GetID()
			if (ID ~= nil and ID >= BANK_BAG_1 and ID <= BANK_BAG_N) then
				for c=1,C_Container.GetContainerNumSlots(ID) do
					local button = _G["ContainerFrame"..t.."Item"..c]
					if (button ~= nil) then
						if (bagResults[ID][c] == 1) then
							button:SetAlpha(1)
						else
							button:SetAlpha(0.15)
						end
					end
				end
			end
		end
	end
end

local function UnfilterGuildBank()
	if (not GuildBankFrame) then return end
	local tab = GetCurrentGuildBankTab()
	for c=1,MAX_GUILDBANK_SLOTS_PER_TAB do
		local index = mod(c, NUM_SLOTS_PER_GUILDBANK_GROUP)
		if (index == 0) then index = NUM_SLOTS_PER_GUILDBANK_GROUP end
		local column = ceil((c-0.5)/NUM_SLOTS_PER_GUILDBANK_GROUP)
		local button = _G["GuildBankFrame"].Columns[column].Buttons[index]
		button:SetAlpha(1)
	end
	local tabs = GetNumGuildBankTabs()
	for t=1,tabs do
		local tab = _G["GuildBankTab"..t].Button
		if (tab.matches) then tab.matches:SetText("") end
	end
end

local function FilterGuildBank()
	if (GuildBankOpen ~= 1 or not GuildBankFrame) then return end
	local text = GuildBankFrame.SearchText:GetText()
	if (text == "Search..." or strlen(text) < 3) then
		if (strlen(text) > 0 and strlen(text) < 3) then
			GuildBankFrame.matches:SetText("Search too narrow")
		else
			GuildBankFrame.matches:SetText("")
		end
		UnfilterGuildBank()
		return
	end
	local filters = {}
	while (1) do
		local a, b, filter = strfind(text,"%%([^%%]+)%%")
		if (filter == nil) then break end
		table.insert(filters,filter)
		text = strsub(text,1,a-1)..strsub(text,b+1)
	end
	if (strfind(text,"%%")) then text = gsub(text,"%%","") end
	local tabs = GetNumGuildBankTabs()
	local results = {}
	local matches = 0
	for t=1,tabs do
		local tabMatches = 0
		results[t] = {}
		for c=1,MAX_GUILDBANK_SLOTS_PER_TAB do
			local ItemLink = GetGuildBankItemLink(t,c)
			if (ItemLink ~= nil) then
				local Item, Link, Quality, ItemLevel, RequiredLevel, itemType, otherType, Stack, equipLoc, Texture, Price = GetItemInfo(ItemLink)
				if (strfind(strlower(Item),"(.*)"..strlower(text).."(.*)") ~= nil) then
					local filterOK = ApplyFilters(ItemLink,filters,t,c,"GB")
					if (filterOK) then
						matches = matches + 1
						tabMatches = tabMatches + 1
						results[t][c] = 1
					end
				end
			end
		end
		local tab = _G["GuildBankTab"..t].Button
		if (not tab.matches) then
			tab.matches = tab:CreateFontString(nil, "OVERLAY")
			tab.matches:SetPoint("TOPLEFT", tab, 30, 14)
			tab.matches:SetJustifyH("LEFT")
			tab.matches:SetFontObject(GameFontNormal)
			tab.matches:SetTextHeight(24)
			tab.matches:SetTextColor(0,1,0)
		end
		if (tabMatches == 0) then tabMatches = "" end
		tab.matches:SetText(tabMatches)
	end
	if (matches == 0) then
		GuildBankFrame.matches:SetText("No results")
		UnfilterGuildBank()
		return
	end
	GuildBankFrame.matches:SetText(matches.." results")
	local tab = GetCurrentGuildBankTab()
	for c=1,MAX_GUILDBANK_SLOTS_PER_TAB do
		local index = mod(c,NUM_SLOTS_PER_GUILDBANK_GROUP)
		if (index == 0) then index = NUM_SLOTS_PER_GUILDBANK_GROUP end
		local column = ceil((c-0.5)/NUM_SLOTS_PER_GUILDBANK_GROUP)
		local button = _G["GuildBankFrame"].Columns[column].Buttons[index]
		if (results[tab][c] ~= 1) then
			button:SetAlpha(0.15)
		else
			button:SetAlpha(1)
		end
	end
end

local LastQuery = ""
StaticPopupDialogs["STACKPACK_SEARCH_BAGS"] = {
	text = "Enter a query of 3 or more characters in length",
	button1 = DONE,
	timeout = 0,
	whileDead = 1,
	hideOnEscape = 1,
	hasEditBox = 1,
	OnShow = function(self)
		_G[self:GetName().."EditBox"]:SetFocus()
		_G[self:GetName().."EditBox"]:SetText(LastQuery)
		_G[self:GetName().."EditBox"]:HighlightText()
	end,
	OnAccept = function()
		UnfilterBags()
	end,
	EditBoxOnTextChanged = function(self)
		local query = _G[self:GetParent():GetName().."EditBox"]:GetText()
		LastQuery = query
		FilterBags(query)
	end,
	EditBoxOnEnterPressed = function(self)
		UnfilterBags()
		self:GetParent():Hide()
	end,
	EditBoxOnEscapePressed = function(self)
		UnfilterBags()
		self:GetParent():Hide()
	end
}

function Stackpack.SearchDialog()
	StaticPopup_Show("STACKPACK_SEARCH_BAGS")
end

local function CheckCTMail()
	local Buffer = nil
	if (CT_MailFrame == nil) then
		Buffer = nil
	else
		if (CT_MailFrame:IsVisible() ~= 1) then
			Buffer = nil
		else
			Buffer = 1
		end
	end
	return Buffer
end

local function Organize()
	local SpecialBags = {}
	for c=1,4 do
		if (C_Container.GetContainerNumSlots(c) > 0 and C_Container.GetContainerNumFreeSlots(c) > 0) then
			local BagID = C_Container.ContainerIDToInventoryID(c)
			local BagLink = GetInventoryItemLink("player",BagID)
			if (BagLink ~= nil) then
				local Item, Link, Quality, ItemLevel, RequiredLevel, itemType, otherType, Stack, equipLoc, Texture, Price = GetItemInfo(BagLink)
				local BagType = select(2, C_Container.GetContainerNumFreeSlots(c))
				if (BagType > 0) then
					local n = #SpecialBags + 1
					SpecialBags[n] = { ["Type"]=BagType, ["Index"]=c, ["BagID"]=BagID }
				end
			end
		end
	end
	if (#SpecialBags == 0) then return end
	for j=1,#SpecialBags do
		for c=BACKPACK_CONTAINER,NUM_TOTAL_EQUIPPED_BAG_SLOTS do
			if (c ~= SpecialBags[j].Index) then
				local i = SpecialBags[j].Index
				local BagType = ""
				if (c > 0) then
					local BagID = C_Container.ContainerIDToInventoryID(c)
					local BagLink = GetInventoryItemLink("player",BagID)
					if (BagLink ~= nil) then
						local Item, Link, Quality, ItemLevel, RequiredLevel, itemType, otherType, Stack, equipLoc, Texture, Price = GetItemInfo(BagLink)
						BagType = select(2, C_Container.GetContainerNumFreeSlots(c))
					end
				end
				if (SpecialBags[j].Type ~= BagType) then
					local slots = C_Container.GetContainerNumSlots(c)
					for s=1,slots do
						local ID = GetItemID(C_Container.GetContainerItemLink(c,s))
						if (ID ~= nil) then
							local Item, Link, Quality, ItemLevel, RequiredLevel, itemType, otherType, Stack, equipLoc, Texture, Price = GetItemInfo(ID)
							local ItemFamily = GetItemFamily(ID)
							if (ItemFamily ~= nil) then
								if (ItemFamily > 0) then
									local FreeSpace = C_Container.GetContainerNumFreeSlots(i)
									if (ItemFamily == SpecialBags[j].Type) then
										C_Container.PickupContainerItem(c,s)
										PutItemInBag(SpecialBags[j].BagID)
										FreeSpace = FreeSpace - 1
									end
									if (FreeSpace <= 0) then break end
								end
							end
						end
					end
				end
			end
		end
	end
end

local function FindStacks(n,c)
	local itemCount = 0
	local itemStack = 0
	local itemID
	local itemName
	if (GuildBankOpen == 1) then
		if (StackpackSettings["StackGuildBank"] or GuildBankStackAll == 1) then
			local Texture, Count, Locked = GetGuildBankItemInfo(n,c)
			if (Count > 0) then
				local ID = GetItemID(GetGuildBankItemLink(n,c))
				if (ID == nil) then return end
				local Item, Link, Quality, ItemLevel, RequiredLevel, itemType, otherType, Stack, equipLoc, Texture, Price = GetItemInfo(ID)
				itemCount = Count
				itemStack = Stack
				itemID = ID
				itemName = Item
			end
		end
	else
		local cInfo = C_Container.GetContainerItemInfo(n,c)
		if (cInfo and (cInfo.quality==-1 or cInfo.stackCount~=nil)) then
			local ID = GetItemID(C_Container.GetContainerItemLink(n,c))
			if (ID ~= nil) then	-- fix bug w/caged pets (rubio)
				local Item, Link, Quality, ItemLevel, RequiredLevel, itemType, otherType, Stack, equipLoc, Texture, Price = GetItemInfo(ID)
				itemCount = cInfo.stackCount
				itemStack = Stack
				itemID = ID
				itemName = Item
			end
		end
	end
	if (itemStack ~= nil) then
		if (itemCount < itemStack) then
			Queued = Queued + 1
			Queue[Queued] = { ["Item"]=itemName,["Bag"]=n,["Slot"]=c,["ID"]=itemID,["Stack"]=itemStack,["Count"]=itemCount,["Need"]=itemStack-itemCount }
		end
	end
end

local function Scan(n)
	if (Enabled == 0 and ForceScan == 0) then return end
	local Slots = 0
	if (GuildBankOpen == 1) then
		Slots = MAX_GUILDBANK_SLOTS_PER_TAB
	else
		Slots = C_Container.GetContainerNumSlots(n)
	end
	for c=1,Slots do
		FindStacks(n,c)
	end
end

local function InitScan()
	local Bags = -1
	local DropBag = -1
	local DropSlot = -1
	local OpenBag = -1
	local OpenSlot = -1
	local StackID = -1
	local FirstBag = 0
	local BagPosition = {}
	local AvailSlots = 0
	Queue = {}
	Queued = -1
	if (BankOpen == 1) then
		FirstBag = BANK_BAG_1
		for c=BANK_BAG_1,BANK_BAG_N do
			if (C_Container.GetContainerNumSlots(c) > 0) then
				Bags = Bags + 1
				BagPosition[Bags+BANK_BAG_1] = c
				local Slots = C_Container.GetContainerNumSlots(c)
				for i=1,Slots do
					local cInfo = C_Container.GetContainerItemInfo(c,i)
					if (cInfo == nil) then
						AvailSlots = AvailSlots + 1
					end
				end
			end
		end
		if (Bags > -1) then
			Bags = Bags + BANK_BAG_1
			if IsReagentBankUnlocked() then
				Bags = Bags + 1
				BagPosition[Bags] = REAGENTBANK_CONTAINER
			end
		end

	elseif (GuildBankOpen == 1) then
		if (StackpackSettings["StackGuildBank"] or GuildBankStackAll == 1) then
			local tab = GetCurrentGuildBankTab()
			if (tab == nil) then
				GuildBankOpen = 0
				return
			end
			tabName, tabIcon, isViewable, canDeposit, numWithdrawals, remainingWithdrawals = GetGuildBankTabInfo(tab)
			if (IsGuildLeader(UnitName("player"))) then canDeposit = 1 end
			if (not canDeposit) then return end
			if (GuildBankStackAll ~= 1) then
				FirstBag = tab
				Bags = tab
				BagPosition[tab] = tab
			else
				FirstBag = 1
				Bags = GetNumGuildBankTabs()
				for c=1,Bags do BagPosition[c]=c end
			end
		end
	else
		Organize()
		FirstBag = 0
		for c=BACKPACK_CONTAINER,NUM_TOTAL_EQUIPPED_BAG_SLOTS do
			if (C_Container.GetContainerNumSlots(c) > 0) then
				Bags = Bags + 1
				BagPosition[Bags] = c
			end
		end
	end
	if (Bags > -1 or BankOpen == 1 or GuildBankOpen == 1) then
		for c=FirstBag,Bags do Scan(BagPosition[c]) end
		if (BankOpen == 1) then
			for c=1,NUM_BANKGENERIC_SLOTS do
				local cInfo = C_Container.GetContainerItemInfo(BANK_CONTAINER,c)
				if (cInfo == nil) then
					AvailSlots = AvailSlots + 1
				end
				FindStacks(BANK_CONTAINER,c)
			end
			BankFrameTitleText:SetText(UnitName("npc").." ("..AvailSlots.." Free Slots)")
		end
		if (Queued > 0) then
			for c=0,Queued do
				for n=0,Queued do
					if (n ~= c) then
						if (Queue[n].ID == Queue[c].ID and Queue[c].ID ~= -1 and Queue[n].ID ~= -1) then
							if (Queue[c].Need <= Queue[n].Need) then
								if (Queue[n].Count < Queue[c].Need) then
									StackID = StackID + 1
									DropBag = Queue[c].Bag
									DropSlot = Queue[c].Slot
									OpenBag = Queue[n].Bag
									OpenSlot = Queue[n].Slot
									Ready[StackID] = { ["Mode"]="All", ["Count"]=Queue[n].Count, ["OpenBag"]=OpenBag, ["OpenSlot"]=OpenSlot, ["DropBag"]=DropBag, ["DropSlot"]=DropSlot }
									Queue[c].Count = Queue[c].Count + Queue[n].Count
									Queue[c].Need = Queue[c].Stack - Queue[c].Count
									Queue[n] = { ["Item"]="",["Bag"]=-1,["Slot"]=-1,["ID"]=-1,["Stack"]=0,["Count"]=0,["Need"]=0 }
								elseif (Queue[n].Count == Queue[c].Need) then
									StackID = StackID + 1
									DropBag = Queue[c].Bag
									DropSlot = Queue[c].Slot
									OpenBag = Queue[n].Bag
									OpenSlot = Queue[n].Slot
									Ready[StackID] = { ["Mode"]="All", ["Count"]=Queue[n].Count, ["OpenBag"]=OpenBag, ["OpenSlot"]=OpenSlot, ["DropBag"]=DropBag, ["DropSlot"]=DropSlot }
									Queue[c] = { ["Item"]="",["Bag"]=-1,["Slot"]=-1,["ID"]=-1,["Stack"]=0,["Count"]=0,["Need"]=0 }
									Queue[n] = { ["Item"]="",["Bag"]=-1,["Slot"]=-1,["ID"]=-1,["Stack"]=0,["Count"]=0,["Need"]=0 }
								elseif (Queue[n].Count > Queue[c].Need) then
									StackID = StackID + 1
									DropBag = Queue[c].Bag
									DropSlot = Queue[c].Slot
									OpenBag = Queue[n].Bag
									OpenSlot = Queue[n].Slot
									Ready[StackID] = { ["Mode"]="Split", ["Count"]=Queue[c].Need, ["OpenBag"]=OpenBag, ["OpenSlot"]=OpenSlot, ["DropBag"]=DropBag, ["DropSlot"]=DropSlot }
									Queue[n].Count = Queue[n].Count - Queue[c].Need
									Queue[n].Need = Queue[n].Stack - Queue[n].Count
									Queue[c] = { ["Item"]="",["Bag"]=-1,["Slot"]=-1,["ID"]=-1,["Stack"]=0,["Count"]=0,["Need"]=0 }
								end
							end
						end
					end
				end
			end
		end
	end
	if (StackID == -1) then
		ForceScan = 0
		if (lastReady>-1) then Push("Stackpack finished") end
		lastReady = StackID
	else
		Stacking = 1
		ReadyID = 0
		State = 0
		if (lastReady > StackID) or (lastReady < 0 and StackID > 5-GuildBankOpen*3) then
			Push("Stackpack is busy ("..StackID..")")
			lastReady = StackID	-- if below threshold this won't be set
		end
		WFrame:SetScript("OnUpdate", StackItem)
	end
end

function QueryScan()	-- is local (forward declared)
	if (not GetCursorInfo() and Stacking == 0) then
		if ((SPToggle == 0 and Enabled == 1) or ForceScan == 1) then
			InitScan()
		end
	end
end

function StackItem()	-- is local (forward declared)
	if (isLooting == true) then return end
	local holding = GetCursorInfo()=="item"
	GuildBankDelay = GuildBankDelay + 1
	if (GuildBankDelay > 250) then State = 3 end
	if (State == 0) then
		if holding then
			GuildBankDelay = 0
			State = 1
		elseif (GuildBankOpen == 1) then
			if (Ready[ReadyID].Mode == "All") then
				PickupGuildBankItem(Ready[ReadyID].OpenBag, Ready[ReadyID].OpenSlot)
			elseif (Ready[ReadyID].Mode == "Split") then
				SplitGuildBankItem(Ready[ReadyID].OpenBag, Ready[ReadyID].OpenSlot, Ready[ReadyID].Count)
			end
		else
			if (Ready[ReadyID].Mode == "All") then
				C_Container.PickupContainerItem(Ready[ReadyID].OpenBag, Ready[ReadyID].OpenSlot)
			elseif (Ready[ReadyID].Mode == "Split") then
				C_Container.SplitContainerItem(Ready[ReadyID].OpenBag, Ready[ReadyID].OpenSlot, Ready[ReadyID].Count)
			end
		end
	end
	if (State == 1) then
		if not holding then
			GuildBankDelay = 0
			State = 3
		elseif (GuildBankOpen == 1) then
			PickupGuildBankItem(Ready[ReadyID].DropBag, Ready[ReadyID].DropSlot)
		else
			C_Container.PickupContainerItem(Ready[ReadyID].DropBag, Ready[ReadyID].DropSlot)
		end
	end
	if (State == 3) then
		ReadyID = ReadyID + 1
		State = 0
		GuildBankDelay = 0
		if (Ready[ReadyID] == nil) then
			WFrame:SetScript("OnUpdate", nil)
			Ready = {}
			Stacking = 0
			ReadyID = 0
			GuildBankStackAll = 0
			Organize()
			InitScan()
		end
	end
end

local function StackGuild_Command(cmd)
	cmd = strlower(cmd)
	if (cmd == "off") then
		StackpackSettings["StackGuildBank"] = false
		Push("Stackpack will not stack the guild bank.")
	elseif (cmd == "on") then
		StackpackSettings["StackGuildBank"] = true
		Push("Stackpack will stack the guild bank. Move an item to begin stacking.")
	elseif (cmd == "help" or cmd == "") then
		Push("/stackguild or /stackgb <on/off> - Toggle stacking of the guild bank.")
	end
end

local function G(setting,style)
	local value = StackpackSettings[setting]
	if (style == 1) then return value and "will" or "will not" end
	if (style == 2) then value = value and "on" or "off"; return "(currently |cffffffff"..value.."|cff646436)" end
	return tostring(value)
end
local function S(cmd,verb,setting)
	local current = StackpackSettings[setting] or false
	local new = not current	-- toggle by default
	-- for a verb of e.g. "ilvl", handle cmds of any sane variant
	-- e.g. "ilvl", "ilvls" to toggle,
	--	"ilvlon", "ilvlson", "ilvl on", "ilvls on" to turn on,
	--	"ilvloff", "ilvlsoff", "ilvl off", "ilvls off" to turn off
	local values = {
		["0"] = false, ["off"] = false, ["no"]  = false, ["false"] = false,
		["1"] = true,  ["on"]  = true,  ["yes"] = true,  ["true"]  = true
	}
	if strsub(cmd,1,strlen(verb)) ~= verb then return false end
	local pos = strlen(verb) + 1
	if strsub(cmd,pos,pos) == "s" then pos = pos + 1 end
	if strsub(cmd,pos,pos) == " " then pos = pos + 1 end
	local value = strsub(cmd,pos)
	if (strlen(value) > 0) then
		new = values[strlower(value)]
		if (new == nil) then
			Push("Bad value "..tostring(value).." for "..verb)
			return true
		end
	end
	StackpackSettings[setting] = new
	return true
end

local function Stackpack_Command(cmd)
	cmd = strlower(cmd)
	if (cmd == "on") then
		Enabled = 1
		Push("Stackpack is on.")
	elseif (cmd == "off") then
		Enabled = 0
		Push("Stackpack is off.")
	elseif (cmd == "scan" or cmd == "start" or cmd == "go" or cmd == "stack") then
		if (GetCursorInfo() or CursorHasItem() or CursorHasMoney() or CursorHasSpell()) then
			Push("Stackpack tells you, \"Please remove the object from your cursor first.\"",2,0.28,2)
		elseif (Stacking == 1) then
			Push("Stackpack tells you, \"I'm busy.. please try again in a few.\"",2,0.28,2)
		else
			ForceScan = 1
			InitScan()
		end
	elseif (cmd == "toggle") then
		if (SPToggle == 0) then
			SplitStack = 0
			SPToggle = 1
			SPCycle = 0
			Push("Stackpack will ignore the next inventory change.")
		else
			SPToggle = 0
			SPCycle = 0
			Push("Stackpack will now resume stacking.")
		end
	elseif (cmd == "searchoff") then
		StackpackSettings["UseInventorySearch"] = 0
		Stackpack_SearchButton:Hide()
		Push("Inventory search disabled.")
	elseif (cmd == "searchon") then
		StackpackSettings["UseInventorySearch"] = 1
		Stackpack_SearchButton:Show()
		Push("Inventory search enabled.")
	elseif (cmd == "search") then
		showSearchDialog()
	elseif S(cmd, "itemid", "ShowItemIDs") then
		Push("Item IDs "..G("ShowItemIDs",1).." be shown in tooltips.")
	elseif S(cmd, "ilvl", "ShowIlvls") then
		Push("Item levels "..G("ShowIlvls",1).." be shown in tooltips.")
	elseif S(cmd, "stacksize", "ShowStacksize") then
		Push("Stack sizes "..G("ShowStacksize",1).." be shown in tooltips.")
	elseif (cmd == "searchhelp") then
		Push("To search the inventory, click the search button (magnifying glass icon) next to your main action bars.")
		Push("If the search button is not visible, try typing /stp searchon to enable inventory searching")
		Push("You can perform filtered searches by entering %SOME_FILTER% in the search box")
		Push("Stackpack supports the following filters:")
		for c=1, #FilterTypes do
			Push("    |cffffffff" .. FilterTypes[c].Type)
		end
		Push("Multiple filters can be given; all must match.")
	elseif (cmd == "help" or cmd == "") then
		Push("Stackpack "..Version.." Help",1,1,0)
		Push("Use /stackpack or /stp to run a command")
		Push("   <go/scan/stack/start> - Tells Stackpack to try and stack items")
		Push("   <on/off> - Turns Stackpack on or off")
		Push("   toggle - Toggles Stackpack on or off momentarily")
		Push("   search - Opens the inventory search dialog")
		Push("   searchhelp - Displays help for inventory searching")
		Push("   searchon - Turns on inventory searching")
		Push("   searchoff - Turns off inventory searching")
		Push("   itemids [on|off] - Turns on/off item IDs in tooltips "..G("ShowItemIDs",2))
		Push("   ilvls [on|off] - Turns on/off item levels in tooltips "..G("ShowIlvls",2))
		Push("   stacksizes [on|off] - Turns on/off stack sizes in tooltips "..G("ShowStacksizes",2))
		Push("/stackguild or /stackgb <on/off> - Toggle stacking of the guild bank "..G("StackGuildBank",2))
	end
end

function Stackpack.OnEvent(self, event, ...)
	local param1 = select(1,...)
	if (event == "PLAYER_INTERACTION_MANAGER_FRAME_HIDE") then
		if	(param1 == Enum.PlayerInteractionType.TradePartner)	then event = "TRADE_CLOSED"
		elseif	(param1 == Enum.PlayerInteractionType.MailInfo)		then event = "MAIL_CLOSED"
		elseif	(param1 == Enum.PlayerInteractionType.Auctioneer)	then event = "AUCTION_HOUSE_CLOSED"
		elseif	(param1 == Enum.PlayerInteractionType.Banker)		then event = "BANKFRAME_CLOSED"
		elseif	(param1 == Enum.PlayerInteractionType.GuildBanker)	then event = "GUILDBANKFRAME_CLOSED"
		end
	elseif (event == "PLAYER_INTERACTION_MANAGER_FRAME_SHOW") then
		if	(param1 == Enum.PlayerInteractionType.Banker)		then event = "BANKFRAME_OPENED"
		elseif	(param1 == Enum.PlayerInteractionType.GuildBanker)	then event = "GUILDBANKFRAME_OPENED"
		end
	end

	if (event == "PLAYER_ENTERING_WORLD") then
		if (StackpackSettings == nil) then StackpackSettings = DefaultSettings end
		if not StackpackSettings["UseInventorySearch"] then Stackpack_SearchButton:Hide() end
		C_Timer.After(2, HookTooltips)	-- try to be last thing in tooltips
		WhatsNew()
	elseif (event == "BAG_UPDATE") then
		if (param1 == nil or param1 < 0 or isLooting == true) then return end
		if (SPToggle == 0) then
			local CTMail = CheckCTMail()
			if (TradeFrame:IsVisible() ~= 1 and SendMailFrame:IsVisible() ~= 1 and CTMail ~= 1) then
				if (AuctionFrame ~= nil) then
					if (AuctionFrame:IsVisible() ~= 1) then
						QueryScan()
					end
				else
					QueryScan()
				end
			end
		else
			SPCycle = SPCycle + 1
			if (SplitStack == 0) then
				if (SPCycle >= 2) then
					SPCycle = 0
					SPToggle = 0
					Push("Stackpack will now resume stacking.")
				end
			else
				if (SPCycle >= 4) then
					SPCycle = 0
					SPToggle = 0
				end
			end
		end
		if (BankOpen == 1) then FilterBank() end
		local dialog = StaticPopup_FindVisible("STACKPACK_SEARCH_BAGS")
		if (dialog) then FilterBags(dialog.editBox:GetText()) end
	elseif (event == "TRADE_CLOSED" or event == "MAIL_CLOSED" or event == "AUCTION_HOUSE_CLOSED") then
		if (not(GetCursorInfo()) and not(CursorHasItem()) and not(CursorHasMoney()) and not(CursorHasSpell()) and Stacking == 0) then
			if (Enabled == 1) then
				InitScan()
			end
		end
	elseif (event == "PLAYERBANKSLOTS_CHANGED") then
		if (SPToggle == 0) then
			QueryScan()
		end
		FilterBank()
	elseif (event == "BANKFRAME_OPENED") then
		BankOpen = 1
		if (SPToggle == 0) then
			QueryScan()
		end
		if (not BankFrame.SearchText) then
			_G["BankFramePurchaseInfo"]:SetPoint("CENTER","BankFrame","CENTER",-23,-95)
			_G["BankFrameSlotCost"]:SetPoint("LEFT","BankFramePurchaseInfo","LEFT",248,0)
			_G["BankFramePurchaseButton"]:SetPoint("RIGHT","BankFramePurchaseInfo","RIGHT",-10,-23)
			BankFrame.SearchText = CreateFrame("EditBox","BankSearchText",BankFrame,"InputBoxTemplate")
			BankFrame.SearchText:SetWidth(130)
			BankFrame.SearchText:SetHeight(13)
			BankFrame.SearchText:SetPoint("TOPLEFT",BankFrame,43,-368)
			BankFrame.SearchText:SetAutoFocus(false)
			BankFrame.SearchText:SetScript("OnTextChanged",FilterBank)
			BankFrame.SearchText:SetScript("OnEditFocusGained",function() BankFrame.SearchText:HighlightText(); BankFrame.SearchText:SetFocus() end)
			BankFrame.SearchText:SetScript("OnEnterPressed",function() if (BankFrame.SearchText:GetText()=="") then BankFrame.SearchText:SetText("Search...") end; BankFrame.SearchText:ClearFocus() end)
		end
		if (not BankFrame.matches) then
			BankFrame.matches = BankFrame:CreateFontString(nil, "OVERLAY")
			BankFrame.matches:SetPoint("TOPLEFT",BankFrame,43,-348)
			BankFrame.matches:SetJustifyH("LEFT")
			BankFrame.matches:SetFontObject(GameFontNormal)
			BankFrame.matches:SetTextHeight(12)
			BankFrame.matches:SetTextColor(0,1,0)
			BankFrame.matches:SetText("")
		end
		BankFrame.SearchText:SetText("Search...")
		BankFrame.SearchText:ClearFocus()
		UnfilterBank()
	elseif (event == "BANKFRAME_CLOSED") then
		BankOpen = 0
		if (SPToggle == 0) then
			QueryScan()
		end
	elseif (event == "GUILDBANKFRAME_OPENED") then
		GuildBankOpen = 1
		if (SPToggle == 0) then
			QueryScan()
		end
		local tabs = GetNumGuildBankTabs()
		if (GuildBankFrame) then
			if (not GuildBankFrame.SearchText) then
				GuildBankFrame.SearchText = CreateFrame("EditBox","GuildBankSearchText",GuildBankFrame,"InputBoxTemplate")
				GuildBankFrame.SearchText:SetWidth(130)
				GuildBankFrame.SearchText:SetHeight(13)
				GuildBankFrame.SearchText:SetPoint("TOPLEFT",GuildBankFrame,31,-45)
				GuildBankFrame.SearchText:SetAutoFocus(false)
				GuildBankFrame.SearchText:SetScript("OnTextChanged",FilterGuildBank)
				GuildBankFrame.SearchText:SetScript("OnEditFocusGained",function() GuildBankFrame.SearchText:HighlightText(); GuildBankFrame.SearchText:SetFocus() end)
				GuildBankFrame.SearchText:SetScript("OnEnterPressed",function() if (GuildBankFrame.SearchText:GetText()=="") then GuildBankFrame.SearchText:SetText("Search...") end; GuildBankFrame.SearchText:ClearFocus() end)
			end
			if (not GuildBankFrame.matches) then
				GuildBankFrame.matches = GuildBankFrame:CreateFontString(nil, "OVERLAY")
				GuildBankFrame.matches:SetPoint("TOPLEFT",GuildBankFrame,165,-45)
				GuildBankFrame.matches:SetJustifyH("LEFT")
				GuildBankFrame.matches:SetFontObject(GameFontNormal)
				GuildBankFrame.matches:SetTextHeight(12)
				GuildBankFrame.matches:SetTextColor(0,1,0)
				GuildBankFrame.matches:SetText("")
			end
			GuildBankFrame.SearchText:SetText("Search...")
			GuildBankFrame.SearchText:ClearFocus()
			UnfilterGuildBank()
		end
		for c=1,tabs do QueryGuildBankTab(c) end
		if (tabs == nil or tabs == 1) then return end
		if (GuildBankStackButton == nil) then
			local HostFrame = GuildBankFrame or _G["BagnonFrameguildbank"]
			if (HostFrame) then
				GuildBankStackButton = CreateFrame("Button", "StackButton", HostFrame, "UIPanelButtonTemplate")
				GuildBankStackButton:SetWidth(130)
				GuildBankStackButton:SetText("Stack Entire Bank")
				if (Bagnon) then
					GuildBankStackButton:SetPoint("BOTTOMLEFT", _G["BagnonGuildTab8"], 33, 5)
				else
					GuildBankStackButton:SetPoint("BOTTOMLEFT", GuildBankFrame, 23, 37)
				end
				GuildBankStackButton:SetScript("OnClick", showStackWarning)
				for c=1, tabs do
					local button = _G["GuildBankTab"..c].Button
					if (Bagnon) then
						button = _G["BagnonGuildTab"..c]
					end
					button:HookScript("OnClick",function() GuildBankStackButton:Show(); FilterGuildBank() end)
				end
			end
		end
	elseif (event == "GUILDBANKFRAME_CLOSED") then
		GuildBankOpen = 0
		if (SPToggle == 0) then
			QueryScan()
		end
	elseif (event == "GUILDBANKBAGSLOTS_CHANGED") then
		if (SPToggle == 0) then
			QueryScan()
		end
		FilterGuildBank()
	elseif (event == "LOOT_OPENED") then
		isLooting = true
	elseif (event == "LOOT_CLOSED") then
		isLooting = false
		QueryScan()
	end
end

function Stackpack.OnLoad(self)
	self:RegisterEvent("PLAYER_ENTERING_WORLD")
	self:RegisterEvent("BAG_UPDATE")
	self:RegisterEvent("PLAYERBANKSLOTS_CHANGED")
	self:RegisterEvent("TRADE_CLOSED")
	self:RegisterEvent("MAIL_CLOSED")
	self:RegisterEvent("AUCTION_HOUSE_CLOSED")
	self:RegisterEvent("BANKFRAME_OPENED")
	self:RegisterEvent("BANKFRAME_CLOSED")
	self:RegisterEvent("GUILDBANKFRAME_OPENED")
	self:RegisterEvent("GUILDBANKFRAME_CLOSED")
	self:RegisterEvent("GUILDBANKBAGSLOTS_CHANGED")
	self:RegisterEvent("LOOT_OPENED")
	self:RegisterEvent("LOOT_CLOSED")
	self:RegisterEvent("PLAYER_INTERACTION_MANAGER_FRAME_HIDE")
	self:RegisterEvent("PLAYER_INTERACTION_MANAGER_FRAME_SHOW")
	SLASH_STACK1 = "/stackpack"
	SLASH_STACK2 = "/stp"
	SLASH_STACKGUILD1 = "/stackguild"
	SLASH_STACKGUILD2 = "/stackgb"
	if (StackSplitOkayButton_OnClick ~= nil) then
		StackHook = StackSplitOkayButton_OnClick
		StackSplitOkayButton_OnClick = function()
			if (StackHook ~= nil) then StackHook() end
			if (BankOpen ~= 1 and GuildBankOpen ~= 1) then
				SplitStack = 1
				SPCycle = 0
				SPToggle = 1
			end
		end
	end
	SlashCmdList["STACK"] = Stackpack_Command
	SlashCmdList["STACKGUILD"] = StackGuild_Command
	Push("|cff646436Stackpack |cffb69d23"..Version.."|cff646436 loaded. Type |cffb69d23/stp help|cff646436 for options.")
end
